@@ -30,7 +30,7 @@
 
 #include "addressbook-migrate.h"
 #include "e-destination.h"
-#include <libebook/e-book-async.h>
+#include <libebook/e-book.h>
 #include <libgnome/gnome-i18n.h>
 #include <gal/util/e-util.h>
 #include <gal/util/e-xml-utils.h>
@@ -300,6 +300,31 @@ migrate_contacts (MigrationContext *context, EBook *old_book, EBook *new_book)
 										"VOICE");
 				attr = attr->next;
 			}
+			/* Replace "POSTAL" (1.4) addresses with "OTHER" (1.5) */
+			else if (!strcmp ("ADR", e_vcard_attribute_get_name (a))) {
+				GList *params, *param;
+				gboolean found = FALSE;
+				EVCardAttributeParam *p;
+
+				params = e_vcard_attribute_get_params (a);
+				for (param = params; param; param = param->next) {
+					p = param->data;
+					if (!strcmp (EVC_TYPE, e_vcard_attribute_param_get_name (p))) {
+						GList *v = e_vcard_attribute_param_get_values (p);
+						if (v && v->data && !strcmp ("POSTAL", v->data)) {
+							found = TRUE;
+							break;
+						}
+					}
+				}
+
+				if (found) {
+					e_vcard_attribute_param_remove_values (p);
+					e_vcard_attribute_param_add_value (p, "OTHER");
+				}
+
+				attr = attr->next;
+			}
 			/* this is kinda gross.  The new vcard parser
 			   needs ';'s to be escaped by \'s.  but the
 			   1.4 vcard generator would put unescaped xml
@@ -673,7 +698,7 @@ migrate_ldap_servers (MigrationContext *context, ESourceGroup *on_ldap_servers)
 }
 
 static ESource*
-get_source_by_uri (ESourceList *source_list, const char *uri)
+get_source_by_name (ESourceList *source_list, const char *name)
 {
 	GSList *groups;
 	GSList *g;
@@ -693,14 +718,9 @@ get_source_by_uri (ESourceList *source_list, const char *uri)
 
 		for (s = sources; s; s = s->next) {
 			ESource *source = E_SOURCE (s->data);
-			char *source_uri = e_source_get_uri (source);
-			gboolean found = FALSE;
+			const char *source_name = e_source_peek_name (source);
 
-			if (!strcmp (uri, source_uri))
-				found = TRUE;
-
-			g_free (source_uri);
-			if (found)
+			if (!strcmp (name, source_name))
 				return source;
 		}
 	}
@@ -757,16 +777,11 @@ migrate_completion_folders (MigrationContext *context)
 						source = e_source_list_peek_source_by_uid (context->source_list, uid);
 				}
 				else {
-					char *uri;
-					char *semi = strchr (physical_uri, ';');
-					if (semi)
-						uri = g_strndup (physical_uri, semi - physical_uri);
-					else
-						uri = g_strdup (physical_uri);
+					char *name = e_xml_get_string_prop_by_name (child, "display-name");
 
-					source = get_source_by_uri (context->source_list, uri);
+					source = get_source_by_name (context->source_list, name);
 
-					g_free (uri);
+					g_free (name);
 				}
 
 				if (source) {
