@@ -34,11 +34,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <glade/glade.h>
 #include <libgnomeui/gnome-stock-icons.h>
-#include <gal/e-table/e-cell-combo.h>
-#include <gal/e-table/e-cell-text.h>
-#include <gal/e-table/e-table-simple.h>
-#include <gal/e-table/e-table-scrolled.h>
-#include <gal/widgets/e-unicode.h>
+#include <gal/util/e-util.h>
 #include <gal/widgets/e-popup-menu.h>
 #include <gal/widgets/e-gui-utils.h>
 #include <widgets/misc/e-dateedit.h>
@@ -47,27 +43,13 @@
 
 #include "../calendar-component.h"
 #include "../e-meeting-attendee.h"
-#include "../e-meeting-model.h"
+#include "../e-meeting-store.h"
+#include "../e-meeting-list-view.h"
 #include "../itip-utils.h"
 #include "comp-editor-util.h"
 #include "e-delegate-dialog.h"
 #include "meeting-page.h"
 
-
-
-enum columns {
-	MEETING_ATTENDEE_COL,
-	MEETING_MEMBER_COL,
-	MEETING_TYPE_COL,
-	MEETING_ROLE_COL,
-	MEETING_RSVP_COL,
-	MEETING_DELTO_COL,
-	MEETING_DELFROM_COL,
-	MEETING_STATUS_COL,
-	MEETING_CN_COL,
-	MEETING_LANG_COL,
-	MEETING_COLUMN_COUNT
-};
 
 /* Private part of the MeetingPage structure */
 struct _MeetingPagePrivate {
@@ -75,7 +57,7 @@ struct _MeetingPagePrivate {
 	GPtrArray *deleted_attendees;
 
 	/* To use in case of cancellation */
-	CalComponent *comp;
+	ECalComponent *comp;
 	
 	/* List of identities */
 	EAccountList *accounts;
@@ -87,16 +69,19 @@ struct _MeetingPagePrivate {
 
 	/* Widgets from the Glade file */
 	GtkWidget *main;
+	GtkWidget *list_box;
 	GtkWidget *organizer_table;
 	GtkWidget *organizer;
 	GtkWidget *existing_organizer_table;
 	GtkWidget *existing_organizer;
 	GtkWidget *existing_organizer_btn;
+	GtkWidget *add;
+	GtkWidget *remove;
 	GtkWidget *invite;
 	
-	/* E Table stuff */
-	EMeetingModel *model;
-	ETableScrolled *etable;
+	/* ListView stuff */
+	EMeetingStore *model;
+	EMeetingListView *list_view;
 	gint row;
 	
 	/* For handling who the organizer is */
@@ -113,10 +98,8 @@ static void meeting_page_finalize (GObject *object);
 
 static GtkWidget *meeting_page_get_widget (CompEditorPage *page);
 static void meeting_page_focus_main_widget (CompEditorPage *page);
-static void meeting_page_fill_widgets (CompEditorPage *page, CalComponent *comp);
-static gboolean meeting_page_fill_component (CompEditorPage *page, CalComponent *comp);
-
-static gint right_click_cb (ETable *etable, gint row, gint col, GdkEvent *event, gpointer data);
+static gboolean meeting_page_fill_widgets (CompEditorPage *page, ECalComponent *comp);
+static gboolean meeting_page_fill_component (CompEditorPage *page, ECalComponent *comp);
 
 static CompEditorPageClass *parent_class = NULL;
 
@@ -178,7 +161,7 @@ meeting_page_init (MeetingPage *mpage)
 	priv->invite = NULL;
 	
 	priv->model = NULL;
-	priv->etable = NULL;
+	priv->list_view = NULL;
 	
 	priv->updating = FALSE;
 }
@@ -215,22 +198,22 @@ get_current_account (MeetingPage *mpage)
 }
 
 static void
-set_attendees (CalComponent *comp, const GPtrArray *attendees)
+set_attendees (ECalComponent *comp, const GPtrArray *attendees)
 {
 	GSList *comp_attendees = NULL, *l;
 	int i;
 	
 	for (i = 0; i < attendees->len; i++) {
 		EMeetingAttendee *ia = g_ptr_array_index (attendees, i);
-		CalComponentAttendee *ca;
+		ECalComponentAttendee *ca;
 		
-		ca = e_meeting_attendee_as_cal_component_attendee (ia);
+		ca = e_meeting_attendee_as_e_cal_component_attendee (ia);
 		
 		comp_attendees = g_slist_prepend (comp_attendees, ca);
 		
 	}
 	comp_attendees = g_slist_reverse (comp_attendees);
-	cal_component_set_attendee_list (comp, comp_attendees);
+	e_cal_component_set_attendee_list (comp, comp_attendees);
 	
 	for (l = comp_attendees; l != NULL; l = l->next)
 		g_free (l->data);	
@@ -243,7 +226,7 @@ cleanup_attendees (GPtrArray *attendees)
 	int i;
 	
 	for (i = 0; i < attendees->len; i++)
-		g_object_unref((g_ptr_array_index (attendees, i)));
+		g_object_unref (g_ptr_array_index (attendees, i));
 }
 
 /* Destroy handler for the task page */
@@ -260,7 +243,7 @@ meeting_page_finalize (GObject *object)
 	priv = mpage->priv;
 
 	if (priv->comp != NULL)
-		g_object_unref((priv->comp));
+		g_object_unref (priv->comp);
 	
 	cleanup_attendees (priv->deleted_attendees);
 	g_ptr_array_free (priv->deleted_attendees, TRUE);
@@ -268,13 +251,13 @@ meeting_page_finalize (GObject *object)
 	if (priv->ia != NULL)
 		g_object_unref (priv->ia);
 	
-	g_object_unref((priv->model));
+	g_object_unref (priv->model);
 	
 	if (priv->main)
 		gtk_widget_unref (priv->main);
 
 	if (priv->xml) {
-		g_object_unref((priv->xml));
+		g_object_unref (priv->xml);
 		priv->xml = NULL;
 	}
 
@@ -335,13 +318,30 @@ clear_widgets (MeetingPage *mpage)
 	priv->existing = FALSE;
 }
 
-/* fill_widgets handler for the meeting page */
 static void
-meeting_page_fill_widgets (CompEditorPage *page, CalComponent *comp)
+sensitize_widgets (MeetingPage *mpage)
+{
+	gboolean read_only;
+	MeetingPagePrivate *priv = mpage->priv;
+
+	if (!e_cal_is_read_only (COMP_EDITOR_PAGE (mpage)->client, &read_only, NULL))
+		read_only = TRUE;
+
+	gtk_widget_set_sensitive (priv->organizer, !read_only);
+	gtk_widget_set_sensitive (priv->existing_organizer_btn, !read_only);
+	gtk_widget_set_sensitive (priv->add, !read_only);
+	gtk_widget_set_sensitive (priv->remove, !read_only);
+	gtk_widget_set_sensitive (priv->invite, !read_only);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->list_view), !read_only);
+}
+
+/* fill_widgets handler for the meeting page */
+static gboolean
+meeting_page_fill_widgets (CompEditorPage *page, ECalComponent *comp)
 {
 	MeetingPage *mpage;
 	MeetingPagePrivate *priv;
-	CalComponentOrganizer organizer;
+	ECalComponentOrganizer organizer;
 	
 	mpage = MEETING_PAGE (page);
 	priv = mpage->priv;
@@ -350,7 +350,7 @@ meeting_page_fill_widgets (CompEditorPage *page, CalComponent *comp)
 	
 	/* Clean out old data */
 	if (priv->comp != NULL)
-		g_object_unref((priv->comp));
+		g_object_unref (priv->comp);
 	priv->comp = NULL;
 	
 	cleanup_attendees (priv->deleted_attendees);
@@ -360,11 +360,11 @@ meeting_page_fill_widgets (CompEditorPage *page, CalComponent *comp)
 	clear_widgets (mpage);
 
 	/* Component for cancellation */
-	priv->comp = cal_component_clone (comp);
+	priv->comp = e_cal_component_clone (comp);
 	
 	/* If there is an existing organizer show it properly */
-	if (cal_component_has_organizer (comp)) {
-		cal_component_get_organizer (comp, &organizer);
+	if (e_cal_component_has_organizer (comp)) {
+		e_cal_component_get_organizer (comp, &organizer);
 		if (organizer.value != NULL) {
 			const gchar *strip = itip_strip_mailto (organizer.value);
 			gchar *string;
@@ -372,19 +372,21 @@ meeting_page_fill_widgets (CompEditorPage *page, CalComponent *comp)
 			gtk_widget_hide (priv->organizer_table);
 			gtk_widget_show (priv->existing_organizer_table);
 			if (itip_organizer_is_user (comp, page->client)) {
-				gtk_widget_show (priv->invite);
-				if (cal_client_get_static_capability (
+				gtk_widget_set_sensitive (priv->invite, TRUE);
+				gtk_widget_set_sensitive (priv->add, TRUE);
+				gtk_widget_set_sensitive (priv->remove, TRUE);
+				if (e_cal_get_static_capability (
 					    page->client,
 					    CAL_STATIC_CAPABILITY_ORGANIZER_NOT_EMAIL_ADDRESS))
 					gtk_widget_hide (priv->existing_organizer_btn);
-				e_meeting_model_etable_click_to_add (priv->model, TRUE);
 			} else {
-				if (cal_client_get_static_capability (
+				if (e_cal_get_static_capability (
 					    page->client,
 					    CAL_STATIC_CAPABILITY_ORGANIZER_NOT_EMAIL_ADDRESS))
 					gtk_widget_hide (priv->existing_organizer_btn);
-				gtk_widget_hide (priv->invite);
-				e_meeting_model_etable_click_to_add (priv->model, FALSE);
+				gtk_widget_set_sensitive (priv->invite, FALSE);
+				gtk_widget_set_sensitive (priv->add, FALSE);
+				gtk_widget_set_sensitive (priv->remove, FALSE);
 			}
 			
 			if (organizer.cn != NULL)
@@ -401,7 +403,7 @@ meeting_page_fill_widgets (CompEditorPage *page, CalComponent *comp)
 		
 		a = get_current_account (mpage);
 		if (a != NULL) {
-			priv->ia = e_meeting_model_add_attendee_with_defaults (priv->model);
+			priv->ia = e_meeting_store_add_attendee_with_defaults (priv->model);
 			g_object_ref (priv->ia);
 
 			e_meeting_attendee_set_address (priv->ia, g_strdup_printf ("MAILTO:%s", a->id->address));
@@ -411,15 +413,19 @@ meeting_page_fill_widgets (CompEditorPage *page, CalComponent *comp)
 	}
 	
 	priv->updating = FALSE;
+
+	sensitize_widgets (mpage);
+
+	return TRUE;
 }
 
 /* fill_component handler for the meeting page */
 static gboolean
-meeting_page_fill_component (CompEditorPage *page, CalComponent *comp)
+meeting_page_fill_component (CompEditorPage *page, ECalComponent *comp)
 {
 	MeetingPage *mpage;
 	MeetingPagePrivate *priv;
-	CalComponentOrganizer organizer = {NULL, NULL, NULL, NULL};
+	ECalComponentOrganizer organizer = {NULL, NULL, NULL, NULL};
 
 	mpage = MEETING_PAGE (page);
 	priv = mpage->priv;
@@ -448,17 +454,17 @@ meeting_page_fill_component (CompEditorPage *page, CalComponent *comp)
 	
 		organizer.value = addr;
 		organizer.cn = a->id->name;
-		cal_component_set_organizer (comp, &organizer);
+		e_cal_component_set_organizer (comp, &organizer);
 
 		g_free (addr);
 	}
 
-	if (e_meeting_model_count_actual_attendees (priv->model) < 1) {
+	if (e_meeting_store_count_actual_attendees (priv->model) < 1) {
 		e_notice (page, GTK_MESSAGE_ERROR,
 			  _("At least one attendee is required."));
 		return FALSE;
 	}
-	set_attendees (comp, e_meeting_model_get_attendees (priv->model));
+	set_attendees (comp, e_meeting_store_get_attendees (priv->model));
 	
 	return TRUE;
 }
@@ -482,6 +488,8 @@ get_widgets (MeetingPage *mpage)
 	if (!priv->main)
 		return FALSE;
 
+	priv->list_box = GW ("list-box");
+
 	/* Get the GtkAccelGroup from the toplevel window, so we can install
 	   it when the notebook page is mapped. */
 	toplevel = gtk_widget_get_toplevel (priv->main);
@@ -497,17 +505,25 @@ get_widgets (MeetingPage *mpage)
 	/* For making the user the organizer */
 	priv->organizer_table = GW ("organizer-table");
 	priv->organizer = GW ("organizer");
-	gtk_combo_set_value_in_list (GTK_COMBO (priv->organizer), TRUE, FALSE);
-	
+	gtk_combo_set_value_in_list (GTK_COMBO (priv->organizer), FALSE, FALSE);
+	gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (priv->organizer)->entry), FALSE);
+
 	/* For showing existing organizers */
 	priv->existing_organizer_table = GW ("existing-organizer-table");
 	priv->existing_organizer = GW ("existing-organizer");
 	priv->existing_organizer_btn = GW ("existing-organizer-button");
+
+	/* Buttons */
+	priv->add = GW ("add-attendee");
+	priv->remove = GW ("remove-attendee");
 	priv->invite = GW ("invite");
 	
 #undef GW
 
-	return (priv->invite
+	return (priv->list_box
+		&& priv->invite
+		&& priv->add
+		&& priv->remove
 		&& priv->organizer_table
 		&& priv->organizer
 		&& priv->existing_organizer_table
@@ -535,10 +551,10 @@ org_changed_cb (GtkWidget *widget, gpointer data)
 			e_meeting_attendee_set_address (priv->ia, g_strdup_printf ("MAILTO:%s", a->id->address));
 			e_meeting_attendee_set_cn (priv->ia, g_strdup (a->id->name));
 			
-			if (!e_meeting_model_find_attendee (priv->model, e_meeting_attendee_get_address (priv->ia), NULL))
-				e_meeting_model_add_attendee (priv->model, priv->ia);
+			if (!e_meeting_store_find_attendee (priv->model, e_meeting_attendee_get_address (priv->ia), NULL))
+				e_meeting_store_add_attendee (priv->model, priv->ia);
 		} else {
-			e_meeting_model_remove_attendee (priv->model, priv->ia);
+			e_meeting_store_remove_attendee (priv->model, priv->ia);
 		}
 	}
 		
@@ -557,120 +573,32 @@ change_clicked_cb (GtkWidget *widget, gpointer data)
 
 	gtk_widget_show (priv->organizer_table);
 	gtk_widget_hide (priv->existing_organizer_table);
-	gtk_widget_show (priv->invite);
-	e_meeting_model_etable_click_to_add (priv->model, TRUE);
+	gtk_widget_set_sensitive (priv->invite, TRUE);
+	gtk_widget_set_sensitive (priv->add, TRUE);
+	gtk_widget_set_sensitive (priv->remove, TRUE);
 
-	comp_editor_page_notify_needs_send (COMP_EDITOR_PAGE (mpage));
+	comp_editor_page_notify_changed (COMP_EDITOR_PAGE (mpage));
 	
 	priv->existing = FALSE;
 }
 
-/* Function called to invite more people */
 static void
-invite_cb (GtkWidget *widget, gpointer data) 
+add_clicked_cb (GtkButton *btn, MeetingPage *mpage)
 {
-	MeetingPage *mpage;
-	MeetingPagePrivate *priv;
+	EMeetingAttendee *attendee;
 	
-	mpage = MEETING_PAGE (data);
-	priv = mpage->priv;
-
-	e_meeting_model_invite_others_dialog (priv->model);
+	attendee = e_meeting_store_add_attendee_with_defaults (mpage->priv->model);
+	e_meeting_list_view_edit (mpage->priv->list_view, attendee);
 }
 
-/* Hooks the widget signals */
 static void
-init_widgets (MeetingPage *mpage)
+remove_attendee (MeetingPage *mpage, EMeetingAttendee *ia) 
 {
 	MeetingPagePrivate *priv;
-
-	priv = mpage->priv;
-
-	/* Organizer */
-	g_signal_connect((GTK_COMBO (priv->organizer)->entry), "changed",
-			    G_CALLBACK (org_changed_cb), mpage);
-
-	g_signal_connect((priv->existing_organizer_btn), "clicked",
-			    G_CALLBACK (change_clicked_cb), mpage);
-
-	/* Invite button */
-	g_signal_connect((priv->invite), "clicked", 
-			    G_CALLBACK (invite_cb), mpage);
-}
-
-#if 0
-static void
-popup_delegate_cb (GtkWidget *widget, gpointer data) 
-{
-	MeetingPage *mpage = MEETING_PAGE (data);
-	MeetingPagePrivate *priv;
-	EDelegateDialog *edd;
-	GtkWidget *dialog;
-	EMeetingAttendee *ia;
-	char *address = NULL, *name = NULL;
-	
-	priv = mpage->priv;
-
-	ia = e_meeting_model_find_attendee_at_row (priv->model, priv->row);
-
-	/* Show dialog. */
-	edd = e_delegate_dialog_new (NULL, itip_strip_mailto (e_meeting_attendee_get_delto (ia)));
-	dialog = e_delegate_dialog_get_toplevel (edd);
-
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK){
-		EMeetingAttendee *ic;
-		
-		name = e_delegate_dialog_get_delegate_name (edd);
-		address = e_delegate_dialog_get_delegate (edd);
-
-		/* Make sure we can add the new delegatee person */
-		if (e_meeting_model_find_attendee (priv->model, address, NULL) != NULL) {
-			e_notice (mpage, GTK_MESSAGE_ERROR,
-				  _("That person is already attending the meeting!"));
-			goto cleanup;
-		}
-		
-		/* Update information for attendee */
-		if (e_meeting_attendee_is_set_delto (ia)) {
-			EMeetingAttendee *ib;
-			
-			ib = e_meeting_model_find_attendee (priv->model, itip_strip_mailto (e_meeting_attendee_get_delto (ia)), NULL);
-			if (ib != NULL) {
-				g_object_ref((ib));
-				g_ptr_array_add (priv->deleted_attendees, ib);
-				
-				e_meeting_model_remove_attendee (priv->model, ib);
-			}			
-		}
-		e_meeting_attendee_set_delto (ia, g_strdup_printf ("MAILTO:%s", address));
-
-		/* Construct delegatee information */
-		ic = e_meeting_model_add_attendee_with_defaults (priv->model);
-		
- 		e_meeting_attendee_set_address (ic, g_strdup_printf ("MAILTO:%s", address));
-		e_meeting_attendee_set_delfrom (ic, g_strdup (e_meeting_attendee_get_address (ia)));
-		e_meeting_attendee_set_cn (ic, g_strdup (name));
-	}
-
- cleanup:
-	g_free (name);
-	g_free (address);
-	g_object_unref((edd));
-}
-#endif
-
-static void
-popup_delete_cb (GtkWidget *widget, gpointer data) 
-{
-	MeetingPage *mpage = MEETING_PAGE (data);
-	MeetingPagePrivate *priv;
-	EMeetingAttendee *ia;
 	int pos = 0;
 	
 	priv = mpage->priv;
 
-	ia = e_meeting_model_find_attendee_at_row (priv->model, priv->row);
-
 	/* If the user deletes the attendee explicitly, assume they no
 	   longer want the organizer showing up */
 	if (ia == priv->ia) {
@@ -682,23 +610,137 @@ popup_delete_cb (GtkWidget *widget, gpointer data)
 	if (e_meeting_attendee_is_set_delfrom (ia)) {
 		EMeetingAttendee *ib;
 		
-		ib = e_meeting_model_find_attendee (priv->model, e_meeting_attendee_get_delfrom (ia), &pos);
-		if (ib != NULL)
+		ib = e_meeting_store_find_attendee (priv->model, e_meeting_attendee_get_delfrom (ia), &pos);
+		if (ib != NULL) {
 			e_meeting_attendee_set_delto (ib, NULL);
+			e_meeting_attendee_set_edit_level (ib,  E_MEETING_ATTENDEE_EDIT_FULL);
+		}		
 	}
 	
 	/* Handle deleting all attendees in the delegation chain */	
 	while (ia != NULL) {
 		EMeetingAttendee *ib = NULL;
 
-		g_object_ref((ia));
+		g_object_ref (ia);
 		g_ptr_array_add (priv->deleted_attendees, ia);
-		e_meeting_model_remove_attendee (priv->model, ia);
+		e_meeting_store_remove_attendee (priv->model, ia);
 
 		if (e_meeting_attendee_get_delto (ia) != NULL)
-			ib = e_meeting_model_find_attendee (priv->model, e_meeting_attendee_get_delto (ia), NULL);
+			ib = e_meeting_store_find_attendee (priv->model, e_meeting_attendee_get_delto (ia), NULL);
 		ia = ib;
 	}
+
+	sensitize_widgets (mpage);
+}
+
+static void
+remove_attendee_at_row (MeetingPage *mpage, int row) 
+{
+	MeetingPagePrivate *priv;
+	EMeetingAttendee *ia;
+	
+	priv = mpage->priv;
+
+	ia = e_meeting_store_find_attendee_at_row (priv->model, row);
+	remove_attendee (mpage, ia);
+}
+
+static void
+remove_clicked_cb (GtkButton *btn, MeetingPage *mpage)
+{
+	MeetingPagePrivate *priv;
+	EMeetingAttendee *ia;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	gboolean valid_iter;
+	char *address;
+	
+	priv = mpage->priv;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->list_view));
+	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+		g_warning ("Could not get a selection to delete.");
+		return;
+	}
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (priv->model), &iter);	
+	
+	gtk_tree_model_get (GTK_TREE_MODEL (priv->model), &iter, E_MEETING_STORE_ADDRESS_COL, &address, -1);
+	ia = e_meeting_store_find_attendee (priv->model, address, NULL);
+	g_free (address);
+	if (!ia)
+		return;
+	
+	remove_attendee (mpage, ia);
+
+	/* Select closest item after removal */
+	valid_iter = gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->model), &iter, path);
+	if (!valid_iter) {
+		gtk_tree_path_prev (path);
+		valid_iter = gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->model), &iter, path);
+	}
+
+	if (valid_iter)
+		gtk_tree_selection_select_iter (selection, &iter);
+
+	gtk_tree_path_free (path);
+}
+
+/* Function called to invite more people */
+static void
+invite_cb (GtkWidget *widget, gpointer data) 
+{
+	MeetingPage *mpage;
+	MeetingPagePrivate *priv;
+	
+	mpage = MEETING_PAGE (data);
+	priv = mpage->priv;
+
+	e_meeting_list_view_invite_others_dialog (priv->list_view);
+}
+
+/* Hooks the widget signals */
+static void
+init_widgets (MeetingPage *mpage)
+{
+	MeetingPagePrivate *priv;
+
+	priv = mpage->priv;
+
+	/* Organizer */
+	g_signal_connect (GTK_COMBO (priv->organizer)->entry, "changed",
+			  G_CALLBACK (org_changed_cb), mpage);
+
+	g_signal_connect (priv->existing_organizer_btn, "clicked",
+			  G_CALLBACK (change_clicked_cb), mpage);
+
+	/* Add attendee button */
+	g_signal_connect (priv->add, "clicked", G_CALLBACK (add_clicked_cb), mpage);
+
+	/* Remove attendee button */
+	g_signal_connect (priv->remove, "clicked", G_CALLBACK (remove_clicked_cb), mpage);
+
+	/* Contacts button */
+	g_signal_connect(priv->invite, "clicked", G_CALLBACK (invite_cb), mpage);
+}
+
+static void
+client_changed_cb (CompEditorPage *page, ECal *client, gpointer user_data)
+{
+	MeetingPage *mpage = MEETING_PAGE (page);
+
+	sensitize_widgets (mpage);
+}
+
+static void
+popup_delete_cb (GtkWidget *widget, gpointer data) 
+{
+	MeetingPage *mpage = MEETING_PAGE (data);
+	MeetingPagePrivate *priv;
+	
+	priv = mpage->priv;
+
+	remove_attendee_at_row (mpage, priv->row);
 }
 
 enum {
@@ -712,66 +754,57 @@ static EPopupMenu context_menu[] = {
 
 	E_POPUP_SEPARATOR,
 #endif
-	E_POPUP_ITEM (N_("_Delete"), G_CALLBACK (popup_delete_cb),   CAN_DELETE),
+	E_POPUP_ITEM (N_("_Remove"), G_CALLBACK (popup_delete_cb),   CAN_DELETE),
 	
 	E_POPUP_TERMINATOR
 };
 
-/* handle context menu over message-list */
 static gint
-right_click_cb (ETable *etable, gint row, gint col, GdkEvent *event, gpointer data)
+button_press_event (GtkWidget *widget, GdkEventButton *event, MeetingPage *mpage)
 {
-	MeetingPage *mpage = MEETING_PAGE (data);
 	MeetingPagePrivate *priv;
 	GtkMenu *menu;
 	EMeetingAttendee *ia;
-	int disable_mask = 0, hide_mask = 0, view_row;
-
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	char *address;
+	int disable_mask = 0, hide_mask = 0;
+	
 	priv = mpage->priv;
 
-	view_row = e_table_model_to_view_row (etable, row);
-	priv->row = e_meeting_model_etable_view_to_model_row (etable, priv->model, view_row);
+	/* only process right-clicks */
+	if (event->button != 3 || event->type != GDK_BUTTON_PRESS)
+		return FALSE;
 
- 	ia = e_meeting_model_find_attendee_at_row (priv->model, priv->row);
+	/* only if we right-click on an attendee */
+	if (!gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (priv->list_view), event->x, event->y, &path, NULL, NULL, NULL))
+		return FALSE;
+	if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->model), &iter, path))
+		return FALSE;
+	
+	gtk_tree_model_get (GTK_TREE_MODEL (priv->model), &iter, E_MEETING_STORE_ADDRESS_COL, &address, -1);
+	ia = e_meeting_store_find_attendee (priv->model, address, &priv->row);
+	g_free (address);
+	if (!ia)
+		return FALSE;
+	
  	if (e_meeting_attendee_get_edit_level (ia) != E_MEETING_ATTENDEE_EDIT_FULL)
  		disable_mask = CAN_DELETE;
  
 	/* FIXME: if you enable Delegate, then change index to '1'.
 	 * (This has now been enabled). */
 	/* context_menu[1].pixmap_widget = gnome_stock_new_with_icon (GNOME_STOCK_MENU_TRASH); */
-	context_menu[1].pixmap_widget =
-	  gtk_image_new_from_stock (GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU);
+	context_menu[0].pixmap_widget =
+	  gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
 
-	menu = e_popup_menu_create (context_menu, disable_mask, hide_mask, data);
+	menu = e_popup_menu_create (context_menu, disable_mask, hide_mask, mpage);
 	e_auto_kill_popup_menu_on_selection_done (menu);
 	
-	gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
-			event->button.button, event->button.time);
+	gtk_menu_popup (menu, NULL, NULL, NULL, NULL, event->button, event->time);
 
 	return TRUE;
 }
 
-
-
-/* Callback used when the ETable gets a focus-out event.  We have to commit any
- * pending click-to-add state for if the event editor is being destroyed.
- */
-static gint
-table_canvas_focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
-{
-	MeetingPage *mpage;
-	MeetingPagePrivate *priv;
-	ETable *etable;
-
-	mpage = MEETING_PAGE (data);
-	priv = mpage->priv;
-
-	etable = e_table_scrolled_get_table (priv->etable);
-
-	e_table_commit_click_to_add (etable);
-	return TRUE;
-}
-
 /**
  * meeting_page_construct:
  * @mpage: An task details page.
@@ -782,35 +815,33 @@ table_canvas_focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer dat
  * be created.
  **/
 MeetingPage *
-meeting_page_construct (MeetingPage *mpage, EMeetingModel *emm,
-			CalClient *client)
+meeting_page_construct (MeetingPage *mpage, EMeetingStore *ems,
+			ECal *client)
 {
 	MeetingPagePrivate *priv;
-	ETable *real_table;
-	gchar *filename;
-	const char *backend_address;
+	char *backend_address;
 	EIterator *it;
 	EAccount *def_account;
 	GList *address_strings = NULL, *l;
+	GtkWidget *sw;
 	
 	priv = mpage->priv;
 
 	priv->xml = glade_xml_new (EVOLUTION_GLADEDIR 
 				   "/meeting-page.glade", NULL, NULL);
 	if (!priv->xml) {
-		g_message ("meeting_page_construct(): "
-			   "Could not load the Glade XML file!");
+		g_message (G_STRLOC ": Could not load the Glade XML file!");
 		return NULL;
 	}
 
 	if (!get_widgets (mpage)) {
-		g_message ("meeting_page_construct(): "
-			   "Could not find all widgets in the XML file!");
+		g_message (G_STRLOC ": Could not find all widgets in the XML file!");
 		return NULL;
 	}
 
 	/* Address information */
-	backend_address = cal_client_get_cal_address (client);
+	if (!e_cal_get_cal_address (client, &backend_address, NULL))
+		return NULL;
 
 	priv->accounts = itip_addresses_get ();
 	def_account = itip_addresses_get_default();
@@ -837,7 +868,8 @@ meeting_page_construct (MeetingPage *mpage, EMeetingModel *emm,
 		}
 	}
 	g_object_unref(it);
-	
+	g_free (backend_address);
+
 	if (address_strings)
 		gtk_combo_set_popdown_strings (GTK_COMBO (priv->organizer), address_strings);
 	else
@@ -848,28 +880,26 @@ meeting_page_construct (MeetingPage *mpage, EMeetingModel *emm,
 	g_list_free (address_strings);
 	
 	/* The etable displaying attendees and their status */
-	g_object_ref((emm));
-	priv->model = emm;
+	g_object_ref (ems);
+	priv->model = ems;
 
-	filename = g_strdup_printf ("%s/config/et-header-meeting-page", evolution_dir);
-	priv->etable = e_meeting_model_etable_from_model (priv->model, 
-							  EVOLUTION_ETSPECDIR "/meeting-page.etspec", 
-							  filename);
-	g_free (filename);
+	priv->list_view = e_meeting_list_view_new (priv->model); 
+	g_signal_connect (G_OBJECT (priv->list_view), "button_press_event", G_CALLBACK (button_press_event), mpage);
 
-	real_table = e_table_scrolled_get_table (priv->etable);
-	g_signal_connect((real_table),
-			    "right_click", G_CALLBACK (right_click_cb), mpage);
-
-	g_signal_connect((real_table->table_canvas), "focus_out_event",
-			    G_CALLBACK (table_canvas_focus_out_cb), mpage);
-
-	gtk_widget_show (GTK_WIDGET (priv->etable));
-	gtk_box_pack_start (GTK_BOX (priv->main), GTK_WIDGET (priv->etable), TRUE, TRUE, 6);
+	gtk_widget_show (GTK_WIDGET (priv->list_view));
+	sw = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_IN);
+	gtk_widget_show (sw);
+	gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (priv->list_view));
+	gtk_box_pack_start (GTK_BOX (priv->list_box), sw, TRUE, TRUE, 6);
 	
 	/* Init the widget signals */
 	init_widgets (mpage);
 
+	g_signal_connect_after (G_OBJECT (mpage), "client_changed",
+				G_CALLBACK (client_changed_cb), NULL);
+
 	return mpage;
 }
 
@@ -882,13 +912,13 @@ meeting_page_construct (MeetingPage *mpage, EMeetingModel *emm,
  * not be created.
  **/
 MeetingPage *
-meeting_page_new (EMeetingModel *emm, CalClient *client)
+meeting_page_new (EMeetingStore *ems, ECal *client)
 {
 	MeetingPage *mpage;
 
 	mpage = g_object_new (TYPE_MEETING_PAGE, NULL);
-	if (!meeting_page_construct (mpage, emm, client)) {
-		g_object_unref((mpage));
+	if (!meeting_page_construct (mpage, ems, client)) {
+		g_object_unref (mpage);
 		return NULL;
 	}
 
@@ -903,7 +933,7 @@ meeting_page_new (EMeetingModel *emm, CalClient *client)
  * 
  * Return value: 
  **/
-CalComponent *
+ECalComponent *
 meeting_page_get_cancel_comp (MeetingPage *mpage)
 {
 	MeetingPagePrivate *priv;
@@ -918,5 +948,5 @@ meeting_page_get_cancel_comp (MeetingPage *mpage)
 	
 	set_attendees (priv->comp, priv->deleted_attendees);
 	
-	return cal_component_clone (priv->comp);
+	return e_cal_component_clone (priv->comp);
 }
